SystemOrganization addCategory: #'Pier-Wysiwyg'!

WAFileLibrary subclass: #PRWysiwygFileLibrary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Wysiwyg'!

!PRWysiwygFileLibrary methodsFor: 'uploaded' stamp: 'NickAger 7/12/2011 10:59'!
parseHtmlToPierWikiSyntaxJs
	^ 'var parseHtmlToPierWikiSyntax = function (startNode) {
	var wikiOutput = [];
	var newLine = "\n"
	
	var surroundingMarkup = function(markup) {
		return function(node) {
			wikiOutput.push(markup);
			walkTheDOM (node.firstChild);
			wikiOutput.push(markup)
		}
	}
	
	var startLineMarkup = function(markup) {
		return function(node) {
			// if not at the start of the document
			if (wikiOutput.length) {
				wikiOutput.push(newLine);
			}
			if (markup.length) {
				wikiOutput.push(markup);
			}
			walkTheDOM (node.firstChild);
			wikiOutput.push(newLine);
		}
	}
	
	var paragraphMarkup = function() {
		return function(node) {
			// if not at the start of the document
			if (wikiOutput.length) {
				wikiOutput.push(newLine);
			}
			if (node.className && node.className.indexOf("annotated") !!= -1) {
				wikiOutput.push("@@");
				wikiOutput.push(node.className.substring(10, node.className.length));
				wikiOutput.push(" ");
			}
			walkTheDOM (node.firstChild);
			wikiOutput.push(newLine);
		}
	}
	
	var listWalk = function () {
		return function (node) {
			var startLength = wikiOutput.length;
			walkTheDOM (node.firstChild);
			var endLength = wikiOutput.length;
			
			// remove consecutive new lines. 
			var previousNewLineIndex = -1;
			for (var i = startLength - 1; i < endLength; ++i) {
				if (wikiOutput [i] == newLine) {
					// we have consecutive new lines.
					if (previousNewLineIndex == i - 1) {
						delete wikiOutput [i];
					}
				previousNewLineIndex = i;		
				}
			}
			
			// ensure we finish with a new line
			if (wikiOutput [wikiOutput.length - 1] && wikiOutput [wikiOutput.length - 1] !!= newLine) {
				wikiOutput.push(newLine);
			}
			
			// ensure there is only a single space between the list markup and the list contents
			for (var i = startLength - 1; i < endLength; ++i) {
				if ((wikiOutput [i] == "# " || wikiOutput [i] == "- ") && (i + 1 < wikiOutput.length)) {
					var listElement = wikiOutput [i + 1];
					var firstNonSpaceIndex = 0;
					while (firstNonSpaceIndex < listElement.length && listElement [firstNonSpaceIndex] == " ") {
						++ firstNonSpaceIndex  ;
					}
					wikiOutput [i + 1] = listElement.substring (firstNonSpaceIndex, listElement.length);
				}
			}		
		}
	}
	
	var listElement = function () {
		return function (node) {
			var parentNode = node;
			while ((parentNode = parentNode.parentNode) && parentNode.nodeName.toLowerCase() !!= "ul" && parentNode.nodeName.toLowerCase() !!= "ol") {}
			var parentNodeName = "";
			if (parentNode) {
				parentNodeName = parentNode.nodeName.toLowerCase();			
			}
			if (parentNodeName == ''ul'') {
				startLineMarkup(''- '')(node)
			}
			if (parentNodeName == ''ol'') {
				startLineMarkup(''# '')(node)
			}
		}
	}
	
	var preMarkup = function () {
		return function (node) {
			wikiOutput.push("=");
			var startLength = wikiOutput.length;
			walkTheDOM (node.firstChild);
			var endLength = wikiOutput.length;
			
			// replace all new lines in text new-line and start of pre-markup
			for (var i = startLength; i < endLength; ++i) {
				wikiOutput [i] = wikiOutput [i].replace(new RegExp(newLine, "g"), newLine+"=");
			}
		}
	}
	
	var tableMarkup = function () {
		return function () {}
	}
	
	var anchor = function(node) {
		return function(node) {
			if(node.getAttribute("href")) {
				var baseUrl = window.location.host;
				// use title rather than href as href can be encoded
				var anchorUrl = node.getAttribute("title");
				var url = (anchorUrl.indexOf(baseUrl) == 0) ? anchorUrl.substr(0, baseUrl.length) : anchorUrl;

				wikiOutput.push("*");
				walkTheDOM(node.firstChild);
				wikiOutput.push(">");
				wikiOutput.push(url);
				wikiOutput.push("*");
			}
			if (node.getAttribute("name")) {
				wikiOutput.push("@");
				wikiOutput.push(node.getAttribute("name"));
			}
		}
	}
	
	// test spans for marking Pier specific markup - currently doesnt do anything special with the markup
	var spanMarkup = function() {
		return function(node) {
			if (node.className && node.className.indexOf("pierNonEditable") !!= -1) {
				walkTheDOM (node.firstChild);
			}
			else {
				walkTheDOM (node.firstChild);
			}
		}
	}

	
	var handleText = function(node) {
		var text = node.nodeValue;

		// if the string starts with a newLine strip it.
		if (text.length && text [0] == newLine) {
			text = text.substr(1, text.length - 1); 
		}		
		
		// if the string ends with a newLine strip it.
		if (text.length && text [text.length - 1] == newLine) {
			text = text.substr(0, text.length - 2); 
		}
		
		if (text.trim().length !!== 0) {
			wikiOutput.push(text);
		}
	}

	var elementToWikiMarkupMappingTable = {
		''b'': surroundingMarkup(''\"\"''),
		''strong'': surroundingMarkup(''\"\"''),
		''i'': surroundingMarkup("\''\''"),
		''em'': surroundingMarkup("\''\''"),
		''ins'': surroundingMarkup("__"),
		''u'': surroundingMarkup("__"),
		"del": surroundingMarkup("--"),
		''code'': surroundingMarkup("=="),
		''sub'': surroundingMarkup("@@"),
		''sup'': surroundingMarkup("^^"),
		"span": spanMarkup(),
		"div": startLineMarkup(''''),
		''a'': anchor(),
		''h1'': startLineMarkup (''!!''),
		''h2'': startLineMarkup(''!!!!''), 
		''h3'': startLineMarkup(''!!!!!!''),
		''h4'': startLineMarkup(''!!!!!!!!''), 
		''h5'': startLineMarkup(''!!!!!!!!!!''), 
		''h6'': startLineMarkup(''!!!!!!!!!!!!''),
		''hr'': startLineMarkup(''_''),
		"br": startLineMarkup(''''),
		"p": paragraphMarkup(),
		"ul": listWalk(),
		"ol": listWalk(),	
		''li'': listElement(),	
		''dl'': startLineMarkup('';''),
		''dd'': startLineMarkup('':''),
		''pre'': preMarkup(),
		''hr'': startLineMarkup("_"),
		''table'': tableMarkup()
	}

    var nodeTypes = {
        ELEMENT_NODE: 1,
        ATTRIBUTE_NODE: 2,
        TEXT_NODE: 3,
        CDATA_SECTION_NODE: 4,
        ENTITY_REFERENCE_NODE: 5,
        ENTITY_NODE: 6,
        PROCESSING_INSTRUCTION_NODE: 7,
        COMMENT_NODE: 8,
        DOCUMENT_NODE: 9,
        DOCUMENT_TYPE_NODE: 10,
        DOCUMENT_FRAGMENT_NODE: 11,
        NOTATION_NODE: 12
    }
	
	// the recursive DOM walker
    var walkTheDOM = function (node) {
		if (node === null)
			return ;
			
		switch (node.nodeType) {
            case nodeTypes.ELEMENT_NODE:
			var nodeName = node.nodeName.toLowerCase();
			if (elementToWikiMarkupMappingTable.hasOwnProperty (nodeName)) {
				elementToWikiMarkupMappingTable [nodeName](node);
			}
			break;
                
            case nodeTypes.TEXT_NODE:
			handleText(node);
			break; 
                
            case nodeTypes.ATTRIBUTE_NODE:
            case nodeTypes.CDATA_SECTION_NODE:
            case nodeTypes.ENTITY_REFERENCE_NODE:
            case nodeTypes.ENTITY_NODE:
            case nodeTypes.PROCESSING_INSTRUCTION_NODE:
            case nodeTypes.COMMENT_NODE:
            case nodeTypes.DOCUMENT_NODE:
            case nodeTypes.DOCUMENT_TYPE_NODE:
            case nodeTypes.DOCUMENT_FRAGMENT_NODE:
            case nodeTypes.NOTATION_NODE:
                break;
			}

		walkTheDOM (node.nextSibling);
    }
	walkTheDOM (startNode.firstChild);
	
	return wikiOutput.join('''');
}' withInternetLineEndings! !

PRViewRenderer subclass: #PRWysiwygHtmlRenderer
	instanceVariableNames: 'wikiRenderer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Wysiwyg'!

!PRWysiwygHtmlRenderer methodsFor: 'private' stamp: 'NickAger 7/11/2011 18:46'!
encodeText: text
	^ String streamContents: [ :stream |
		(WAXmlEncoder on: stream codec: GRNullCodec new) nextPutAll: text]! !

!PRWysiwygHtmlRenderer methodsFor: 'constants' stamp: 'NickAger 7/11/2011 17:35'!
nonEditable
	^ 'yui-noedit pierNonEditable'
	
! !

!PRWysiwygHtmlRenderer methodsFor: 'visiting-document' stamp: 'NickAger 7/12/2011 08:41'!
visitHeader: anObject
	"override as in the editor we don't care about the named anchor"
	html heading level: anObject level; with: [
		self visitDocumentGroup: anObject ]! !

!PRWysiwygHtmlRenderer methodsFor: 'visiting-document' stamp: 'NickAger 7/12/2011 08:07'!
visitInternalLink: aLink
	aLink isEmbedded 
		ifTrue: [
			html span
				class: 'pierEmbeddedLink';
				class: self nonEditable;
					with: [
						html text: (self encodeText: (self writeWikiOutputFor: aLink)) ] ]	
		ifFalse: [
			super visitInternalLink: aLink ]

! !

!PRWysiwygHtmlRenderer methodsFor: 'visiting-document' stamp: 'NickAger 7/12/2011 08:07'!
visitValueLink: aLink 
	html span
		class: 'pierValueLink';
		class: self nonEditable;
			with: [
				html text: (self encodeText: (self writeWikiOutputFor: aLink)) ]
! !

!PRWysiwygHtmlRenderer methodsFor: 'visiting-document' stamp: 'NickAger 7/12/2011 07:54'!
visitVerbatim: aVerbatim
	html span
		class: 'pierVerbatim';
		class: self nonEditable;
			with: [
				html text: (self encodeText: (self writeWikiOutputFor: aVerbatim)) ] ! !

!PRWysiwygHtmlRenderer methodsFor: 'accessing' stamp: 'NickAger 7/11/2011 18:56'!
writeWikiOutputFor: anObject
	^ String streamContents: [ :aStream | PRWikiWriter new write: anObject to: aStream ]
	! !

WAComponent subclass: #PRWysiwygEditor
	instanceVariableNames: 'pierWikiText tabs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pier-Wysiwyg'!

!PRWysiwygEditor class methodsFor: 'initialize-release' stamp: 'NickAger 7/7/2011 17:20'!
initialize
	super initialize.
	self register! !

!PRWysiwygEditor class methodsFor: 'initialize-release' stamp: 'NickAger 7/7/2011 17:19'!
register
	(WAAdmin register: self asApplicationAt: 'wysiwygtest')
		addLibrary: JQDeploymentLibrary;
		addLibrary: JQUiDeploymentLibrary;
		addLibrary: JQRedmondTheme;
		preferenceAt: #scriptGeneratorClass put: JQScriptGenerator.! !

!PRWysiwygEditor methodsFor: 'rendering' stamp: 'NickAger 7/11/2011 17:42'!
editorJs: textAreaId
	"see: 
		* http://developer.yahoo.com/yui/examples/editor/toolbar_editor.html
		* http://developer.yahoo.com/yui/docs/editor.js.html
		* http://developer.yahoo.com/yui/editor/
		* http://developer.yahoo.com/yui/examples/editor/cal_editor.html
		* http://developer.yahoo.com/yui/examples/editor/flickr_editor.html"
	| selectPierLinksHtml |
	selectPierLinksHtml := 
		WAHtmlCanvas builder
			fullDocument: false;
			scriptGeneratorClass: JQScriptGenerator; "removes the onload from body"				
			render: [:html | self renderSiteMapSelectOn: html id: textAreaId, '_createlink_sitemap' ].
	
	selectPierLinksHtml replaceAll: $" with: $'.	"subsitute double quotes for single quotes"

	^ 
'var myEditor = new YAHOO.widget.Editor("', textAreaId, '", {
	height: "300px",
	width: "600px",
	autoHeight: true,
	allowNoEdit: true,
	extracss: ".pierNonEditable { border: 1px dashed; display:inline-block;}",
	toolbar: {
		focusAtStart: true,
		buttons: [ 
	    		{ group: "parastyle", label: "",
				buttons: [
					{ type: "select", label: "Normal", value: "heading", disabled: true,
						menu: [
							{ text: "Normal", value: "none", checked: true },
							{ text: "Header 1", value: "h1" },
							{ text: "Header 2", value: "h2" },
							{ text: "Header 3", value: "h3" },
							{ text: "Header 4", value: "h4" },
							{ text: "Header 5", value: "h5" },
							{ text: "Header 6", value: "h6" }
						]
					}
				]
			},
			{ type: "separator" },
			{ group: "indentlist", label: "",
			    buttons: [
			        { type: "push", label: "Create an Unordered List", value: "insertunorderedlist" },
			        { type: "push", label: "Create an Ordered List", value: "insertorderedlist" }
			    ]
			},
			{ type: "separator" },
			{ group: "textstyle" , label: "",
				buttons: [
            			{ type: "push", label: "Bold CTRL + SHIFT + B",  value: "bold" }, 
					{ type: "push", label: "Italic CTRL + SHIFT + I", value: "italic" }, 
					{ type: "push", label: "Underline CTRL + SHIFT + U", value: "underline" },
					{ type: "push", label: "Code", value: "code" },
					{ type: "push", label: "Strikeout", value: "strike" },
					{ type: "separator" },
					{ type: "push", label: "Subscript", value: "subscript", disabled: true },
					{ type: "push", label: "Superscript", value: "superscript", disabled: true },
				 ]
			 },
	    		{ type: "separator" },
			{ group: "insertitem", label: "",
			    buttons: [
				{ type: "push", label: "Horizontal Rule", value: "rule" },
			        { type: "push", label: "HTML Link CTRL + SHIFT + L", value: "createlink", disabled: true },
			        { type: "push", label: "Insert Image", value: "insertimage" }
			    ]
			}
		]
	}
} );

myEditor._renderCreateLinkWindow = function() {
var Dom = YAHOO.util.Dom,
    Event = YAHOO.util.Event,
    Lang = YAHOO.lang;

	var str = "<label for=''" + this.get("id") + "_createlink_url''><strong>" + this.STR_LINK_URL + ":</strong> <input type=''text'' name=''" + this.get("id") + "_createlink_url'' id=''" + this.get("id") + "_createlink_url'' value=''''></label>";
	str += "<label for=''" + this.get("id") + "_createlink_sitemap''><strong>Site-map:</strong>', selectPierLinksHtml, '</label>";
	str += "<label for=''" + this.get("id") + "_createlink_target''><strong>&nbsp;</strong><input type=''checkbox'' name=''" + this.get("id") + "_createlink_target'' id=''" + this.get("id") + "_createlink_target'' value=''_blank'' class=''createlink_target''> " + this.STR_LINK_NEW_WINDOW + "</label>";
	str += "<label for=''" + this.get("id") + "_createlink_title''><strong>" + this.STR_LINK_TITLE + ":</strong> <input type=''text'' name=''" + this.get("id") + "_createlink_title'' id=''" + this.get("id") + "_createlink_title'' value=''''></label>";

	var body = document.createElement("div");
	body.innerHTML = str;

	var unlinkCont = document.createElement("div");
	unlinkCont.className = "removeLink";
	var unlink = document.createElement("a");
	unlink.href = "#";
	unlink.innerHTML = this.STR_LINK_PROP_REMOVE;
	unlink.title = this.STR_LINK_PROP_REMOVE;
	Event.on(unlink, "click", function(ev) {
		Event.stopEvent(ev);
		this.unsubscribeAll("afterExecCommand");
		this.execCommand("unlink");
		this.closeWindow();
	}, this, true);
	unlinkCont.appendChild(unlink);
	body.appendChild(unlinkCont);

	this._windows.createlink = {};
	this._windows.createlink.body = body;
	//body.style.display = "none";
	Event.on(body, "keyup", function(e) {
		Event.stopPropagation(e);
	});
	this.get("panel").editor_form.appendChild(body);
	this.fireEvent("windowCreateLinkRender", { type: "windowCreateLinkRender", panel: this.get("panel"), body: body });
	return body;
}


myEditor._handleCreateLinkClick = function () {
var Dom = YAHOO.util.Dom,
    Event = YAHOO.util.Event,
    Lang = YAHOO.lang;
            var el = this._getSelectedElement();
            if (this._isElement(el, "img")) {
                this.STOP_EXEC_COMMAND = true;
                this.currentElement[0] = el;
                this.toolbar.fireEvent("insertimageClick", { type: "insertimageClick", target: this.toolbar });
                this.fireEvent("afterExecCommand", { type: "afterExecCommand", target: this });
                return false;
            }
            if (this.get("limitCommands")) {
                if (!!this.toolbar.getButtonByValue("createlink")) {
                    return false;
                }
            }
            
            this.on("afterExecCommand", function() {
                var win = new YAHOO.widget.EditorWindow("createlink", {
                    width: "350px"
                });
                
                var el = this.currentElement[0],
                    url = "",
                    title = "",
                    target = "",
                    localFile = false;
                if (el) {
                    win.el = el;
                    if (el.getAttribute("href", 2) !!== null) {
                        url = el.getAttribute("href", 2);
                        if (this._isLocalFile(url)) {
                            //Local File throw Warning
                            win.setFooter(this.STR_LOCAL_FILE_WARNING);
                            localFile = true;
                        } else {
                            win.setFooter(" ");
                        }
                    }
                    if (el.getAttribute("title") !!== null) {
                        title = el.getAttribute("title");
                    }
                    if (el.getAttribute("target") !!== null) {
                        target = el.getAttribute("target");
                    }
                }
                var body = null;
                if (this._windows.createlink && this._windows.createlink.body) {
                    body = this._windows.createlink.body;
                } else {
                    body = this._renderCreateLinkWindow();
                }

                win.setHeader(this.STR_LINK_PROP_TITLE);
                win.setBody(body);

		    Event.purgeElement(this.get("id") + "_createlink_url");

                Dom.get(this.get("id") + "_createlink_url").value = url;
                Dom.get(this.get("id") + "_createlink_title").value = title;
                Dom.get(this.get("id") + "_createlink_target").checked = ((target) ? true : false);
                

                Event.onAvailable(this.get("id") + "_createlink_url", function() {
                    var id = this.get("id");
                    window.setTimeout(function() {
                        try {
                            YAHOO.util.Dom.get(id + "_createlink_url").focus();
                        } catch (e) {}
                    }, 50);

                    if (this._isLocalFile(url)) {
                        //Local File throw Warning
                        Dom.addClass(this.get("id") + "_createlink_url", "warning");
                        this.get("panel").setFooter(this.STR_LOCAL_FILE_WARNING);
                    } else {
                        Dom.removeClass(this.get("id") + "_createlink_url", "warning");
                        this.get("panel").setFooter(" ");
                    }
                    Event.on(this.get("id") + "_createlink_url", "blur", function() {
                        var url = Dom.get(this.get("id") + "_createlink_url");
                        if (this._isLocalFile(url.value)) {
                            //Local File throw Warning
                            Dom.addClass(url, "warning");
                            this.get("panel").setFooter(this.STR_LOCAL_FILE_WARNING);
                        } else {
                            Dom.removeClass(url, "warning");
                            this.get("panel").setFooter(" ");
                        }
                    }, this, true);
                }, this, true);
                
                this.openWindow(win);

            });
        }


myEditor.cmd_code = function (o) {
	this.execCommand("inserthtml", "<code>" + this._getSelection() + "</code>");
}
myEditor.cmd_rule = function (o) {
	this.execCommand("inserthtml", "<hr>");
}
myEditor.cmd_strike = function (o) {
	this.execCommand("inserthtml", "<del>" + this._getSelection() + "</del>");
}

myEditor.render();
' withInternetLineEndings

"	
{ group: ''formatting'' , label: ''Formatting'',
  buttons: [
	{ type: ''separator'' },
	{ type: ''push'', label: ''Remove Formatting'', value: ''removeformat'', disabled: true },
	{ type: ''push'', label: ''Show/Hide Hidden Elements'', value: ''hiddenelements'' }
   ]
}
"
! !

!PRWysiwygEditor methodsFor: 'updating' stamp: 'NickAger 7/8/2011 06:06'!
jQueryTabsCustomisationUpdateRoot: anHtmlRoot
		anHtmlRoot stylesheet add:  '
.ui-widget {
	font-size: 13px;
}'! !

!PRWysiwygEditor methodsFor: 'rendering' stamp: 'NickAger 7/12/2011 10:37'!
onSelectTabJSOn: html
	^ 
	(JSStream on: 'if (ui.index == 0) {') ,
	((html jQuery id: #hiddenWikiText) value: ((html jQuery id: #standardPierEditor) value)),
	(html jQuery ajax serialize: (html jQuery id: #hiddenWikiText)), 
	((JSStream on: '}	
if (ui.index == 1) {') ,
	(JSStream on: 'myEditor.setEditorHTML(myEditor.cleanHTML())'),
	((html jQuery id: #hiddenWikiText) value: (JSStream on: 'parseHtmlToPierWikiSyntax(myEditor._getDoc().body)')),
	(html jQuery ajax serialize: (html jQuery id: #hiddenWikiText)),
	(JSStream on: '}')) asFunction: #('event' 'ui')! !

!PRWysiwygEditor methodsFor: 'accessing' stamp: 'NickAger 7/12/2011 10:16'!
pierWikiText
	^ pierWikiText ifNil: [ 
		pierWikiText := '!!A Wysiwyg editor for Pier

Currently the editor supports:
# ""Bold""
# ''''Italic''''
# __Underline__
# ==code==
# --strikeout--
# Headings
# line-breaks and paragraphs
# ordered and unordered lists.
# super^^script^^ and sub@@script@@
# horizontal line

_
The translator also supports the following, although there are no editor buttons to support it:
- definition list and definitions
- preformated.
- named anchors
- email addresses: *mail me>nick.ager@gmail.com*
- annotated paragraphs
@ananchor

@@note an annotated paragraph

Still to come support for tables. Support for internal links still needs to be expanded.

!!!!!! Support verbatim, embedded links and value links
The editor attempts to support all the Pier Wiki markup, so the Wysiwyg editor can be used interchangeably with the standard editor without fear of loosing any markup so, for example, it should be possible to use for editing Pier book content. I''ve taken the approach of preserving verbatim, embedded and value links in the Wysiwyg editor, but not making them editable. The editor could be extended with custom dialogs in which the raw wiki syntax could be edited.

Some non-editable wiki content:
{{{<div class="pierIcon">verbatim test</div}}}
+value:time+
_

!!!!!! Thoughts and questions
Some features should only be enabled for users with sufficient permissions. For example you don''t want untrusted users to be able to insert verbatium html or edit the content in the wiki standard view. However, is it acceptable to make the Wysiwyg editor dependant on Pier-Security? 
I propose we enable/disable features based on the following permissions:
- ==PRVerbatiumMarkupPermission==
- ==PRWikiTextEditingPermission==
- ==PREmbeddedLinksPermission==
- ==PRValueLinksPermission==

!!!!!!Useful links
- *http://developer.yahoo.com/yui/editor/*
- *http://yuiblog.com/assets/pdf/cheatsheets/editor.pdf*
- *http://developer.yahoo.com/yui/examples/editor/toolbar_editor.html*
- *http://developer.yahoo.com/yui/docs/editor.js.html*
- *http://developer.yahoo.com/yui/examples/editor/cal_editor.html*
- *http://developer.yahoo.com/yui/examples/editor/flickr_editor.html*']! !

!PRWysiwygEditor methodsFor: 'accessing' stamp: 'NickAger 7/5/2011 13:36'!
pierWikiText: anObject
	pierWikiText := anObject! !

!PRWysiwygEditor methodsFor: 'rendering' stamp: 'NickAger 7/12/2011 07:52'!
renderContentOn: html
	html form: [
		self renderTabsOn: html.
		self renderHiddenWikiTextOn: html ]
! !

!PRWysiwygEditor methodsFor: 'rendering' stamp: 'NickAger 7/7/2011 17:42'!
renderHiddenWikiTextOn: html
	html hiddenInput
		id: #hiddenWikiText;
		on: #pierWikiText of: self
! !

!PRWysiwygEditor methodsFor: 'rendering' stamp: 'NickAger 7/11/2011 12:24'!
renderPierWikiTextToHtmlOn: html
	| parsedStructure renderer |
		
	parsedStructure := PRDocumentParser parseStream: self pierWikiText readStream.
	renderer := PRWysiwygHtmlRenderer new.
	renderer withinContentDo: [
		renderer start: parsedStructure in: self on: html ]! !

!PRWysiwygEditor methodsFor: 'rendering' stamp: 'NickAger 7/8/2011 16:40'!
renderSiteMapSelectOn: html id: id
	| kernel user viewableStructures structurePaths |
	
	"I want to see how the site map appears to a non-admin user, so add myself to the kernel if I'm not already there"
	kernel := PRKernel instances anyOne.
	user :=  kernel userNamed: 'nick.ager@gmail.com' ifNone: [ kernel users add: ((PUUser named: 'nick.ager@gmail.com') password: 'apassword'; yourself) ].
	viewableStructures := kernel root enumerator with; all; select: [ :aStructure | aStructure securityDecoration hasPermission: (PUPermission on: PRViewCommand) for: user].
	
	structurePaths := viewableStructures collect: [ :aStructure | aStructure absolutePath].
	structurePaths := (Array with:  'OR select a location Url from site-map ...') , structurePaths.
	
	html select
		id: id;
		list: structurePaths! !

!PRWysiwygEditor methodsFor: 'rendering' stamp: 'NickAger 7/7/2011 17:37'!
renderStandardPierEditorOn: html
	"the standard pier editor toolbar expects the textarea to be wrapped in a td - yuk"
	html table
		cellpadding: 0;
		cellspacing: 0;
		with: [
			html tableRow: [
				html tableData
					class: 'wiki';
					with: [
						html textArea
							id: #standardPierEditor;
							script: (JSStream on: 'EditorEnh($("#standardPierEditor")[0], { binding: true, toolbar: true})');
							with: self pierWikiText ] ] ]! !

!PRWysiwygEditor methodsFor: 'rendering' stamp: 'NickAger 7/7/2011 17:27'!
renderTabsOn: html
	html div
		script: (html jQuery this tabs
			onSelect: (self onSelectTabJSOn: html));
		with: [
			html unorderedList: [
				self tabs associationsDo: [ :each |
					html listItem: [
						html anchor
							url: (html jQuery ajax html: each value; fullUrl);
							with: each key ] ] ] ]
! !

!PRWysiwygEditor methodsFor: 'rendering' stamp: 'NickAger 7/11/2011 12:22'!
renderWysiwygEditorOn: html
	| textAreaId |
	
	html textArea
		id: (textAreaId := html nextId);
		with: [ self renderPierWikiTextToHtmlOn: html ].
		
	html script: (self editorJs: textAreaId)
! !

!PRWysiwygEditor methodsFor: 'updating' stamp: 'NickAger 7/7/2011 17:08'!
standardPierEditorUpdateRoot: anHtmlRoot
	PREditorScripts default updateRoot: anHtmlRoot.! !

!PRWysiwygEditor methodsFor: 'private' stamp: 'NickAger 7/7/2011 17:18'!
tabs
	^ tabs ifNil: [
		tabs := Dictionary new.
		tabs 
			at: 'Wysiwyg editor' put: [ :r | self renderWysiwygEditorOn: r ];
			at: 'Standard Pier Editor' put: [ :r | self renderStandardPierEditorOn: r ].
		tabs ]! !

!PRWysiwygEditor methodsFor: 'updating' stamp: 'NickAger 7/11/2011 17:34'!
updateRoot: anHtmlRoot
	super updateRoot: anHtmlRoot.
	
	self wysiwygEditorUpdateRoot: anHtmlRoot.
	self standardPierEditorUpdateRoot: anHtmlRoot.
	
	self jQueryTabsCustomisationUpdateRoot: anHtmlRoot! !

!PRWysiwygEditor methodsFor: 'updating' stamp: 'NickAger 7/8/2011 05:53'!
wysiwygEditorCustomisationUpdateRoot: anHtmlRoot
	anHtmlRoot bodyAttributes addClass: 'yui-skin-sam'.
	anHtmlRoot javascript url: PRWysiwygFileLibrary / #parseHtmlToPierWikiSyntaxJs.
	
	anHtmlRoot stylesheet add: '
.yui-toolbar-group {
	margin-top: 4px;
}
.yui-skin-sam .yui-toolbar-container .yui-toolbar-code span.yui-toolbar-icon {
	background-image: url( /files/PREditorScripts/toolbar.png );
	background-position: -64px 0px;
	margin-left: 5px;
	margin-top: 2px;
}

.yui-skin-sam .yui-toolbar-container .yui-toolbar-rule span.yui-toolbar-icon {
	background-image: url( /files/PREditorScripts/toolbar.png );
	background-position: -208px 0px;
	margin-left: 5px;
	margin-top: 2px;
}

.yui-skin-sam .yui-toolbar-container .yui-toolbar-strike span.yui-toolbar-icon {
	background-image: url( /files/PREditorScripts/toolbar.png );
	background-position: -48px 0px;
	margin-left: 5px;
	margin-top: 2px;
}
'
	
! !

!PRWysiwygEditor methodsFor: 'updating' stamp: 'NickAger 7/8/2011 16:35'!
wysiwygEditorStandardConfigUpdateRoot: anHtmlRoot
	anHtmlRoot stylesheet url: 'http://yui.yahooapis.com/2.9.0/build/assets/skins/sam/skin.css'.
	
	anHtmlRoot javascript url: 'http://yui.yahooapis.com/2.9.0/build/yahoo-dom-event/yahoo-dom-event.js'.
	anHtmlRoot javascript url: 'http://yui.yahooapis.com/2.9.0/build/element/element-min.js'.
	
	"Needed for Menus, Buttons and Overlays used in the Toolbar"
	anHtmlRoot javascript url: 'http://yui.yahooapis.com/2.9.0/build/container/container_core-min.js'.
	anHtmlRoot javascript url: 'http://yui.yahooapis.com/2.9.0/build/menu/menu-min.js'.
	anHtmlRoot javascript url: 'http://yui.yahooapis.com/2.9.0/build/button/button-min.js'.
	
	"Source file for Rich Text Editor"
	anHtmlRoot javascript url: 'http://yui.yahooapis.com/2.9.0/build/editor/editor-min.js'.	

	"non minimised versions"
	"anHtmlRoot javascript url: 'http://yui.yahooapis.com/2.9.0/build/yahoo/yahoo.js'.	
	anHtmlRoot javascript url: 'http://yui.yahooapis.com/2.9.0/build/dom/dom.js'.	
	anHtmlRoot javascript url: 'http://yui.yahooapis.com/2.9.0/build/event/event.js'.
		
	anHtmlRoot javascript url: 'http://yui.yahooapis.com/2.9.0/build/element/element.js'.
	
	anHtmlRoot javascript url: 'http://yui.yahooapis.com/2.9.0/build/container/container_core.js'.
	anHtmlRoot javascript url: 'http://yui.yahooapis.com/2.9.0/build/menu/menu.js'.
	anHtmlRoot javascript url: 'http://yui.yahooapis.com/2.9.0/build/button/button.js'.
	
	anHtmlRoot javascript url: 'http://yui.yahooapis.com/2.9.0/build/editor/editor.js'.	"
! !

!PRWysiwygEditor methodsFor: 'updating' stamp: 'NickAger 7/8/2011 05:53'!
wysiwygEditorUpdateRoot: anHtmlRoot.
	"See YUI editor documentation: 
		* http://developer.yahoo.com/yui/editor/
		* http://yuiblog.com/assets/pdf/cheatsheets/editor.pdf"

	self wysiwygEditorStandardConfigUpdateRoot: anHtmlRoot.
	self wysiwygEditorCustomisationUpdateRoot: anHtmlRoot.
! !
PRWysiwygEditor initialize!
